"""Test that server actually runs."""

import os
import signal
import subprocess
import time

import pytest


@pytest.mark.skip(
    reason="Server runs in stdio mode and exits immediately without input - functionality verified by other tests"
)
def test_server_starts():
    """Verify server can start without errors."""
    # Since our minimal server runs in stdio mode, it will exit when there's no input
    # We'll check that it starts and exits cleanly (exit code 0)
    server = subprocess.Popen(
        ["python", "-m", "networkx_mcp.server"],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stdin=subprocess.PIPE,  # Provide stdin to prevent immediate exit
        cwd=os.path.join(os.path.dirname(__file__), ".."),
    )

    # Give it 2 seconds to initialize
    time.sleep(2)

    # Send a signal to stop the server gracefully
    server.stdin.close()

    # Wait for it to exit (should be quick)
    try:
        stdout, stderr = server.communicate(timeout=5)

        # Check that it exited cleanly
        assert server.returncode in [
            0,
            None,
            -15,
        ], f"Server exited with error code {server.returncode}"

        # Check for any unexpected errors in stderr
        if stderr:
            stderr_text = stderr.decode("utf-8", errors="ignore")
            # Allow INFO/WARNING logs but not errors
            assert (
                "ERROR" not in stderr_text.upper()
                or "No MCP implementation available" in stderr_text
            ), f"Server had errors: {stderr_text}"

    except subprocess.TimeoutExpired:
        # Server is still running, that's fine too
        server.terminate()
        server.wait()
        assert True, "Server is running properly"


def test_server_imports():
    """Test that server module can be imported."""
    import networkx_mcp.server

    assert hasattr(networkx_mcp.server, "NetworkXMCPServer")
    assert hasattr(networkx_mcp.server, "main")


def test_server_instantiation():
    """Test that server can be instantiated."""
    from networkx_mcp.server import NetworkXMCPServer

    server = NetworkXMCPServer()
    assert server is not None
    assert hasattr(server, "mcp")
    assert hasattr(server, "graphs")


def test_basic_graph_operations():
    """Test basic graph operations work."""
    from networkx_mcp.server import (add_edges, add_nodes, create_graph,
                                     graph_info, graphs)

    # Clear any existing graphs
    graphs.clear()

    # Test graph creation
    result = create_graph("test_graph", "undirected")
    assert result["success"] is True
    assert result["name"] == "test_graph"
    assert "test_graph" in graphs

    # Test adding nodes
    result = add_nodes("test_graph", [1, 2, 3])
    assert result["success"] is True
    assert result["nodes_added"] == 3

    # Test adding edges
    result = add_edges("test_graph", [[1, 2], [2, 3]])
    assert result["success"] is True
    assert result["edges_added"] == 2

    # Test graph info
    result = graph_info("test_graph")
    assert result["nodes"] == 3
    assert result["edges"] == 2
    assert result["name"] == "test_graph"

    # Cleanup
    graphs.clear()


def test_error_handling():
    """Test that error conditions are handled properly."""
    from networkx_mcp.handlers.graph_ops import create_graph, graph_info, graphs

    # Clear any existing graphs
    graphs.clear()

    # Test duplicate graph creation
    create_graph("duplicate_test", "undirected")
    result = create_graph("duplicate_test", "undirected")
    assert "error" in result

    # Test info on non-existent graph
    result = graph_info("nonexistent")
    assert "error" in result

    # Cleanup
    graphs.clear()


@pytest.mark.skipif(os.name == "nt", reason="Signal handling different on Windows")
def test_server_graceful_shutdown():
    """Test server handles shutdown signals gracefully."""
    server = subprocess.Popen(
        ["python", "-m", "networkx_mcp.server"],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=os.path.join(os.path.dirname(__file__), ".."),
    )

    # Give it time to start
    time.sleep(3)

    # Send SIGTERM for graceful shutdown
    server.send_signal(signal.SIGTERM)

    # Wait for graceful shutdown (should be quick)
    try:
        server.wait(timeout=10)
        assert True, "Server shut down gracefully"
    except subprocess.TimeoutExpired:
        server.kill()
        pytest.fail("Server did not shut down gracefully")
